@@ -1,5 +1,6 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
+from __future__ import division
 from decimal import Decimal
 from sql import Table
 
@@ -45,7 +46,10 @@ class Uom(ModelSQL, ModelView):
         help=('The coefficient for the formula:\n'
             'coef (base unit) = 1 (this unit)'))
     rounding = fields.Float('Rounding Precision', digits=(12, 12),
-        required=True, states=STATES, depends=DEPENDS)
+        required=True, states=STATES, depends=DEPENDS,
+        domain=[
+            ('rounding', '>', 0),
+            ])
     digits = fields.Integer('Display Digits', required=True)
     active = fields.Boolean('Active')
 
@@ -106,29 +110,36 @@ class Uom(ModelSQL, ModelView):
     @fields.depends('factor')
     def on_change_factor(self):
         if (self.factor or 0.0) == 0.0:
-            return {'rate': 0.0}
-        return {
-            'rate': round(1.0 / self.factor, self.__class__.rate.digits[1]),
-            }
+            self.rate = 0.0
+        else:
+            self.rate = round(1.0 / self.factor, self.__class__.rate.digits[1])
 
     @fields.depends('rate')
     def on_change_rate(self):
         if (self.rate or 0.0) == 0.0:
-            return {'factor': 0.0}
-        return {
-            'factor': round(1.0 / self.rate, self.__class__.factor.digits[1]),
-            }
+            self.factor = 0.0
+        else:
+            self.factor = round(
+                1.0 / self.rate, self.__class__.factor.digits[1])
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        return ['OR',
+        if clause[1].startswith('!') or clause[1].startswith('not '):
+            bool_op = 'AND'
+        else:
+            bool_op = 'OR'
+        return [bool_op,
             (cls._rec_name,) + tuple(clause[1:]),
             ('symbol',) + tuple(clause[1:]),
             ]
 
     @staticmethod
     def round(number, precision=1.0):
-        return round(number / precision) * precision
+        i, d = divmod(precision, 1)
+        base = round(number / precision)
+        # Instead of multiply by the decimal part, we must divide by the
+        # integer to avoid precision lost due to float
+        return (base * i) + ((base / (1 / d)) if d != 0 else 0)
 
     @classmethod
     def validate(cls, uoms):
